## Instructions: Sequential, Conditional or Iterative

This quiz is designed to measure your understanding of instructions in pseudo-code. Your task is to determine the type of instruction and indicate its effect.

Passing grade: 80%

Quiz type: Instructional

Q1: In a car, there is a sensor that measures the current speed of the vehicle (stored in the variable "speed"). To control the speed of the car, this speed is compared to a maximum speed (stored in the variable "maxspeed"). An indicator light shows the driver the current situation: it is green if the vehicle is going below the maximum speed; otherwise it is red. How can the situation be represented using an alternative control instruction?

Question type: 
- Multiple correct answers

Answers:

```
If (speed > maxspeed) then Turn on red light 
Turn on green light 
```
Wrong -> These instructions will be executed, but will not produce the desired result: if speed is greater than maxspeed, the indicator light will turn red, then the instruction immediately following the *if* will be executed, turning the indicator light green. If speed is under maxspeed, the instruction to turn on the red light will not be executed and the instruction immediately following the *if* will be executed, turning the indicator light green. So regardless of speed, the green light will be on. 

```
If (speed > maxspeed) then Turn on red light
Else Turn on green light
```
Correct -> These instructions will be executed and will produce the desired result. The speed is being correctly tested (here, the interval of speeds satisfying the condition is [maxspeed, +∞]) and the actions dependent on it are declared.

```
If (speed < maxspeed) or speed = maxspeed) then Turn on green light
Else Turn on red light 
```
Correct -> These instructions will be executed and will produce the desired result. Indeed, the speed is being correctly tested (here, the interval of speeds satisfying the condition is [maxspeed, +∞]) and the actions dependent on it are declared.

```
If (speed > maxspeed) then Turn on red light
Else If (speed < maxspeed or speed = maxspeed) then Turn on green light
```
Correct -> These instructions will be executed and will produce the desired result. However, they are considered incorrect because the second instruction is superfluous. If the *else* is executed, then the condition speed > maxspeed must be false, and therefore the condition speed < maxspeed or speed = maxspeed is necessarily true. The second *if* is therefore useless.


Q2: Consider the equation ax + b = 0. 

Here is the proposed process for calculating the value of x:
```
Declare variable a
Declare variable b
Declare variable x
Obtain a keyboard value for a
Obtain a keyboard value for b
Assign to x the result of b ÷ a
Display the value of x onscreen
```
This process will fail in certain situations. How can this be corrected?

Question type: 
- Multiple correct answers

Answers:

```
Declare variable a
Declare variable b
Declare variable x
If (a = 0) then Obtain a keyboard value for a
Obtain a keyboard value for b
Assign to x the result of b ÷ a
Display the value of x onscreen
```
Wrong -> This solution will not calculate the value of x correctly. The value of a is tested before having been entered with the keyboard.

```
Declare variable a
Declare variable b
Declare variable x
Obtain a keyboard value for a
Obtain a keyboard value for b
If (a does not = 0) then Assign to x the result of b ÷ a
Display the value of x onscreen
```
Wrong -> This solution does not determine the value of x in all cases. In the particular case of a = 0, x will not be calculated.


```
Declare variable a
Declare variable b
Declare variable x
Obtain a keyboard value for a
Obtain a keyboard value for b
If (a = 0) then Display the message "No solution or All possible values of x are solutions if the value of b is 0"
Else Assign to x the result of b ÷ a
Display the value of x onscreen
```

Correct -> The test applied to a is correct and this process determines the value of x in all cases; in no event will the program have to calculate b ÷ 0, which is impossible.

```
Declare variable a
Declare variable b
Declare variable x
Obtain a keyboard value for a
Obtain a keyboard value for b
If (a does not = 0) then Assign to x the result of b ÷ a
Else Display the value of x onscreen

```
Wrong -> The value of x will be calculated if a is not 0, but it will not be displayed. Conversely, it will be displayed but not calculated if the value of a is 0.
```
Declare variable a
Declare variable b
Declare variable x
Obtain a keyboard value for a
If (a = 0) then Display the message “No solution”
Else { Obtain a keyboard value for b
      Assign to x the result of b ÷ a
      Display the value of x onscreen
      }
      
      The { and } symbols demarcate the instructions to be executed by the *Else* loop.
```
Correct -> Yes, this solution works. If a = 0, then no solution is possible. Otherwise, the value of b can be used to find x, by dividing b by a then displaying the value of x.


Q3: A number is chosen at random – it is possible to generate a number randomly – and a player has to guess it. The number’s value is stored in the variable "guessme". How would you design a process that verifies whether the player has succeeded (assuming the player is allowed an infinite number of attempts)? 

Question type: 
- Single correct answer 

Answers:

```
Declare variable guessme
Declare variable selection
Pick a value at random for guessme
Obtain a keyboard value for selection
While (selection does not = guessme) Display the message "Try again"
Display the message "You win"
```
Wrong -> If the keyboard selection is wrong, the loop will continually display the message "Try again" because the condition remains true.

```
Declare variable guessme
Declare variable selection
Pick a value for guessme
Obtain a keyboard value for selection
If (selection does not = guessme) then Display the message "Try again"
Else Display the message "You win"
```
Wrong -> This solution will only test one single attempt. No other attempts are possible if the player’s first selection is wrong.

```
Declare variable guessme
Declare variable selection
Pick a value at random for guessme
Obtain a keyboard value for selection
While (selection does not = guessme) {
          Display the message "Try again"
          Obtain a keyboard value for selection
          Display the message "You win"
          }
          
The { and } symbols demarcate the instructions to be repeated in the *While* loop
```
Correct -> Yes, this solution enables successive selections by the player and tests the value until it matches the randomly-picked value.

Q4: To make stiff egg whites, you have to beat them with a whisk until the bowl can be held upside down without the egg whites slipping out. 

Question type: 
- Single correct answer 

Answers:
Correct/Wrong -> Feedback if selected

```
If (egg whites slip?) then Beat egg whites once
```
Wrong -> With this solution, the egg whites will only be beaten once.
```
While (egg whites slip?) Beat the egg whites once
```
Correct -> This solution will cause the eggs to be beaten over and over until they no longer slip.
```

While (egg whites slip?) 
    If (egg whites slip?) then Beat the egg whites once
```

Correct -> This solution works but is nonetheless considered incorrect, because the *If* test embedded in the *While* loop is superfluous. If the *While* loop is executed, that means the “eggs white slip?” condition is true. Therefore, it is useless to test it with *If* before beating the eggs once.

```
If (egg whites slip?) then 
    While (egg whites slip?) Beat the egg whites once
```
Correct -> This solution works but is nonetheless considered incorrect, because the *If* test is superfluous. It is pointless to test whether the egg whites are slipping before the *While* loop, which tests the condition from the outset.