## Problem-solving

This quiz is designed to measure your understanding of how to implement and execute a program in order to solve a given problem.

Passing grade: 80%

Quiz type: Instructional/Certifying

You are asked to write a program that will determine the price (inclusive of taxes) for a certain number of product units, based on the product’s unit price and category. The program must select one of two value-added tax (VAT) rates (7.7% or 2.5%), depending on the product category (the lower rate gets applied to food items, non-alcoholic beverages, books, newspapers, magazines and pharmaceuticals). You are told that each of the VAT rates could potentially change over time.

Q1: In order to process the request correctly, the initial data you need are:

both VAT rates 
the pre-tax unit price of the product 
the unit count for the product

Wrong -> the product category is missing. The program must determine the applicable VAT rate based on the product category. This data must therefore be known.

both VAT rates 
the pre-tax unit price of the product 
the unit count for the product 
the product category

Correct ->  with this data set, you have everything you need to calculate the final price (inclusive of taxes).

both VAT rates 
the pre-tax unit price of the product 
the unit count for the product 
the product name 
the product category

Wrong -> the product name is not used in this process. Whenever you are defining variables, it’s important to avoid unnecessary variables. Even though every product does have a name, this information is not used in this process. The category suffices. It’s possible to create a more ambitious solution that would identify product category based on product name, but then the "category" variable would be superfluous.

a variable for the product’s pre-tax unit price 
a variable for the number of product units 
a variable for the product category

Wrong ->  even if you know both VAT rates beforehand, this data needs to be available to the program. You could use the VAT rates directly in order to compute an answer, but you have been told these rates can change. In such a case, it’s more effective to use two variables whose values can be adjusted if the rates are ever changed.

Q2: What constitutes the intermediate data?

There is none.

Wrong ->  A required piece of data is missing in order to calculate the total price (inclusive of taxes):
The VAT rate to be applied.

Correct ->  Initially, both VAT rates are known. But the program must know which one to apply its calculation to.

The number of product units.

Wrong -> the number of product units cannot be calculated; it is part of the initial data that must be obtained when launching the program.

The applicable VAT rate 
The total pre-tax price

Correct -> the calculation can be carried out in two stages: first the total pre-tax price, then the total price inclusive of taxes. In practice, it’s best to limit the number of variables as much as possible. Though correct, this is not the most efficient solution.

Q3: Which of the following constitute output data?

The total price, all taxes included

Correct ->  Yes, the desired outcome is the total price, all taxes included.

The applicable VAT rate 
The total pre-tax price

Correct ->  Even though the amount of VAT is not explicitly required, it is calculated during this process and it may be worthwhile to communicate it to the user in addition to the total price.

The product's unit price 
The total price, all taxes included

Wrong -> the product’s unit price is a piece of initial data. It is already known to the user. You may remind the user of it at the end of the process, but this will not be considered an output or result.

Q4: Which variable name is most appropriate for the product’s unit price?

a

Wrong -> variable names should be selected in such as way as to indicate their role in the process; this makes them easier to use.

The_product_unit_price

Wrong ->  variable names should be selected in such a way as to indicate their role in the process, but it is best to make them as short as possible.

Unit_price

Correct: this name is both concise and reflective of the variable’s role in the process.

U_price

Wrong: the meaning of "U" in this variable name is unclear (albeit not too difficult to guess). It is best to be as explicit as possible.

Q5: What are the appropriate initialization actions for the input data?

Enter both VAT rates 
Enter the product’s unit price 
Enter the product category

Wrong -> both VAT rates are known and do not need to be entered.

Assign the value of each VAT rate 
Enter the product’s unit price 
Enter the product category

Correct ->  Both VAT rates are known, so their values can be directly assigned to the relevant variables. The product’s unit price and the product category cannot be known or calculated, so they must be entered by the program user (in real-life conditions, the price and product category would be determined using a bar-code on the product’s packaging).

Assign the value of each VAT rate 
Enter the product’s unit price

Wrong -> the product category cannot be calculated or inferred from the other data. It must therefore be entered by the user.

Q6: What are the main processes/computations of this program and what is their proper order?

Compute the final price, all taxes included 
Determine the applicable VAT rate

Wrong ->  In order to compute the price with all taxes included, the applicable VAT rate must be known. It must therefore be determined beforehand.

Determine the applicable VAT rate 
Compute the final price, all taxes included

Correct ->  The final price with all taxes included is calculated using the unit price, the number of units and the VAT rate. First, unit price and number of units are entered by the user at the beginning of the program. Then the applicable VAT rate is determined. Finally, the price computed.

Compute the final price, all taxes included

Wrong -> in order to compute the price with all taxes included, the applicable VAT rate must be known. There are two possible rates. The applicable rate must therefore be determined before any calculation can take place.

Q7: How is the applicable VAT rate determined?

While (rate is not the right one) change the rate.

Wrong -> there are two possible VAT rates. Selection of the right rate is based on a single criteria: product category. A loop is not called for here.

If (category = food items and non-alcoholic beverages and books and newspapers and magazines and pharmaceuticals) then use lower rate 
Else use regular rate

Wrong -> Using *and* would set the product category variable to all categories at once, which is impossible.

If (category = food items or non-alcoholic beverages or books or newspapers or magazines or pharmaceuticals) then use lower rate 
Else use regular rate

Correct ->  An *if then else* statement allows us to test the value of the category variable and select which of the rates to apply. If the product category matches any of the categories listed as a lower-rate product, the lower VAT will be applied.

If (category = food items or non-alcoholic beverages or books or newspapers or magazines or pharmaceuticals) then use regular rate 
Else use lower rate

Wrong -> If the condition is true, it is the lower rate, not the regular rate, that needs to be applied. 

If (category = food items and non-alcoholic beverages and books and newspapers and magazines and pharmaceuticals) then use regular rate
Else use lower rate

Correct ->  An *if then else* statement allows us to test the value of the category variable and select which of the rates to apply. If the product does not belong to any of the categories listed as a lower-rate product, the regular VAT rate will be applied.