# Case study: Tic-Tac-Toe

## Problem statement

Write a program for two players to play tic-tac-toe together 
(https://en.wikipedia.org/wiki/Tic-tac-toe). 

The game is played turn by turn. It uses a 3x3 grid.

Each player is assigned a symbol: either O or X. The players take turns entering 
their symbol in the grid’s cells. The winner is whoever is able to 
line up three identical symbols, whether horizontally, vertically or diagonally.

It is possible for neither player to win. 

Once the grid is full (no moves are available) or a player wins, the program must display 
one of two messages: either "Game over: Player … wins" or "Game over: it’s a draw".

Your program must be written using methods.

## Solution

The game is on a grid, so we’ll need an array of size 3x3 to contain the symbols entered 
by the players. We’ll use a *while* loop to ask the players to enter 
a move, display the array, verify that neither player has won and, lastly, 
make it the next player’s turn.  This means we’ll need to write methods to perform the following tasks:

- Display the array
- Check whether the array contains three identical symbols horizontally, vertically or diagonally, meaning a player has won
- Randomly pick which player gets to move first 
- Define which player's turn is next
- Enter the player’s move in the grid

Last instruction: all your methods must be under *main* and used inside a *while* loop.


# Code production: Tic-Tac-Toe

Your task is to define and code several methods in order to create a 
game of tic-tac-toe, using your work from the previous case study and its solution.


## Solution


```scala
import java.lang.Boolean
import io.StdIn

object game {
    def main(args: Array[String]): Unit = {
        var matrix = Array.ofDim[String](3,3)
        var tab = Array.fill(10)(0)
        var winner = false

        println("Welcome to Tic-Tac-Toe 3000")

        display(matrix)

        var player = playsFirst()
        println("Player " + player + " has the first move.")

        while(winner == false){
            println("Your move, " + player)
            val x = StdIn.readLine("Coordinate x: ").toInt
            val y = StdIn.readLine("Coordinate y: ").toInt

            matrix = move(player,x,y,matrix)
            display(matrix)
            winner = check(matrix)
            player = whoseTurn(player)
        }
        println("Game Over")
        display(matrix)
    }
    
    def display(mat:Array[Array[String]]): Unit={
        for{i <- 0 until 3}{ // row
            for{j <- 0 until 3}{ // column
                print("|" + mat(i)(j) + "|")  
            }
        println()
        }
    }
  
    def whoseTurn(player:String):String={
        if(player == "X"){
            return "O"
        } 
        else {
            return "X"
        }
    }
  
    def playsFirst():String={
    
        val random = new scala.util.Random
        val plrs = Array("X","O")
        return plrs(random.nextInt(2))
    }
  
    def move(player:String,i:Int,j:Int,mat:Array[Array[String]]):Array[Array[String]]={
        if(mat(i)(j) == null){
            mat(i)(j)=  player
            return mat
        }
        else {
            return mat
        }
    }
  
    def check(mat:Array[Array[String]]): Boolean ={
        val linedUp =  Array(Array("XXX"),Array("OOO"))
        
        for (x <- mat){
            if(x.mkString.contains(linedUp(0).mkString) || x.mkString.contains(linedUp(1).mkString)){
                return true
            }
        }
        
        for (x <- mat.transpose){
            if(x.mkString.contains(linedUp(0).mkString) || x.mkString.contains(linedUp(1).mkString)){
                return true
            }
        }

        for{i <- 0 until 3}{
            for{j <- 0 until 3}{
                if(mat(i)(j) == null){
                    return false
                }
            }

        }
        return false
    }
}
```
